/*
TALKTHRU_INT.ASM
B.BOUCHEZ - June 2002
Note : this code is provided 'as is', with no warranty and/or support
You are free to use it as you want
*/

#define AC97_BUFSIZE    2             /* 1 sample gauche / 1 sample droite */
#define DMAPAGE         0x0C          /* DMA Page Register */
#define TX0ADDR         0x48          /* TX0 DMA Address */
#define TX0NXTADDR      0x49          /* TX0 DMA Next Address */
#define TX0CNT          0x4A          /* TX0 DMA Count */
#define TX0CURCNT       0x4B          /* TX0 DMX Current Count */
#define RX0ADDR         0x4C          /* RX0 DMA Address */
#define RX0NXTADDR      0x4D          /* RX0 DMA Next Address */
#define RX0CNT          0x4E          /* RX0 DMA Count */
#define RX0CURCNT       0x4F          /* RX0 DMX Current Count */

#define AC97LCTL        0xC0          /* AC'97 control */
#define AC97LSTAT       0xC2          /* AC'97 status */
#define AC97SEN         0xC4          /* AC'97 slot enable */

#define STCTL0			0x10		  /* FIFO 0 TX ctrl Reg */			
#define SRCTL0			0x11		  /* FIFO 0 RX ctrl reg */

#include "ac97def.h"

.section/pm IVtable;
    jump start; nop; nop; nop;        /* Reset */
    rti; nop; nop; nop;               /* Powerdown */
    rti; nop; nop; nop;               /* Kernel - Single step */
    rti; nop; nop; nop;               /* Stack status */
    rti; nop; nop; nop;               /* Mailbox */
    rti; nop; nop; nop;               /* Timer */
    rti; nop; nop; nop;               /* Reserved (ring) */
    rti; nop; nop; nop;               /* PCI Bus Master */
    rti; nop; nop; nop;               /* DSP - DSP */
    rti; nop; nop; nop;               /* FIFO 0 Transmit */
    jump input_irq; nop; nop; nop;               /* FIFO 0 Receive */
    rti; nop; nop; nop;               /* FIFO 1 Transmit */
    rti; nop; nop; nop;               /* FIFO 1 Receive */
    rti; nop; nop; nop;               /* Reserved (INT 13) */    
    rti; nop; nop; nop;               /* Reserved (INT 14) */
    rti; nop; nop; nop;               /* AC'97 frame */
    
.section/dm data1;
.var    in_buffer[AC97_BUFSIZE];
.var    out_buffer[AC97_BUFSIZE];

.section/pm program;
start:

/* Initialisation page DMA */
    ax0 = 0;                          /* Page DMA = 0 */
    reg(DMAPAGE) = ax0;
    
/* Configuration DMA buffer de sortie */
    ax0 = out_buffer;                 /* Adresse du buffer */
    reg(TX0ADDR) = ax0;
    reg(TX0NXTADDR) = ax0;
    ax0 = AC97_BUFSIZE;
    reg(TX0CNT) = ax0;
    reg(TX0CURCNT) = ax0;
    
/* Configuration DMA buffer d'entre */
    ax0 = in_buffer;                 /* Adresse du buffer */
    reg(RX0ADDR) = ax0;
    reg(RX0NXTADDR) = ax0;
    ax0 = AC97_BUFSIZE;
    reg(RX0CNT) = ax0;
    reg(RX0CURCNT) = ax0;

/* AC'97 configuration */
/* STCTL0 : 0000 1001 0011 1010
TU, TFE, TFF, Reserved = 0000
DME = 1 : DMA Enabled
FIP [2:0] = 111 : Interrupt quand 2 mots vides dans FIFO
SLOT[3:0] = 0011 : Slots 3/4 (PCM L/R)
SMSEL = 1 : Stereo
Reserved : 0
CE1:0 = 10 : connect to AC'97
*/
    ax0 = 0x0f3a;
    reg(STCTL0) = ax0;
    
/* SRCTL0 : 0000 1000 0011 1010
RO, RFE, RFF, Reserved = 0000
DME = 1 : DMA Enabled
FIP [2:0] = 001 : Interrupt quand 2 mots reu dans FIFO
SLOT[3:0] = 0011 : Slots 3/4 (PCM L/R)
SMSEL = 1 : Stereo
Reserved : 0
CE1:0 = 10 : connect to AC'97
*/
    ax0 = 0x83a;
    reg(SRCTL0) = ax0;
    
/* ACLink Activation */
/* AC97LCTL : 0000 0000 1110 0001
00000 : Reserved
ARPD = 0 : No automatic AC97 powerdown
AGPE = 0 : AC'97 GPIO Disable (Slot 12 disabled)
ACWE = 0 : AC'97 lonk on AC'97 wake event
LKEN = 1 : AC'97 Link Enable
BCEN = 1 : Bitclk internally generated
BCOE = 1 : Bitclk output enable
SSPM = 0 : Simple Serial Port Mode = 0 (AC'97 mode)
AFD = 0 : SDO drives serial data stream out
AFS = 0 : SYNC pin high during slot 0
AFR = 0 : AC'97 not resetted
SYEN = 1 : SYNC enable (data exchange over AC'97)
*/
    iopg=0;
    ax0=0x00e1;
    io(AC97LCTL)=ax0;

/* Wait for AC'97 */
    ay0 = 0x2000;    /* ACR[2:0] = 001 : codec 1 prt */
wait_codec1:
    ax0 = io(AC97LSTAT);
    ar = ax0 and ay0;
    if eq jump wait_codec1;
    
/* slots 3 and 4 = ON */
    ax0 = 0x1800;
    io(AC97SEN) = ax0;
    
/* Codec setup */
    iopg = 4;          /* Codec 1 */
    ax0 = 0x0303;
    io(AC97_AUX_OUT_VOL) = ax0;    /* Right headphone vol : 0dB, Left headphone vol : 0 dB */
    
    ax0 = 0x0000;
    io(AC97_PCM_OUT_VOL) = ax0;    /* PCM Out Volume */
    
    ax0 = 0x0404;
    io(AC97_REC_SEL) = ax0;    /* Record Selector : Line In */
    
    ax0 = 0x0000;
    io(AC97_REC_GAIN) = ax0;    /* Record gain : O dB */

	ena int;
//	imask = 0x0400;	   /* FIFO0 receive (for stand alone application) */
	imask = 0x0414;		/* FIFO0 Receive+Kernel+Mailbox (for Visual DSP debug kernel) */

/* Main loop */
main_loop:
    jump main_loop;

/* IRQ routine */
input_irq:
	ena sec_reg;
	ax0 = dm(in_buffer);
	dm(out_buffer)=ax0;
	ar = dm(in_buffer+1);
	dm(out_buffer+1)=ar;
	rti;

